## 责任链埋点思路

### 埋点需求

行为分析埋点通常需要包括某一事件发生时的前因、后果，以及事件发生对象的特征。在复杂的数据分析、模型训练等需求中，不仅仅需要获知某个事件的发生次数，对埋点上下文尤为关注。此处上下文指的通常有 2 类，分别是：

- 事件发生的页面信息和页面位置信息
- 用户经过怎样的路径来到当前页面，也就是“来源”信息

下面结合具体的西瓜视频场景，看一个简单的埋点需求，“点击收藏”事件

![图片](https://p3-juejin.byteimg.com/tos-cn-i-k3u1fbpfcp/85f841476fd74b33ad4caa095bc2ceea~tplv-k3u1fbpfcp-zoom-1.image)

作为最常见的消费场景，列表和详情都有收藏按钮，产品希望知道每一个收藏事件发生的场景，方便后续优化收藏功能，以及结合用户收藏的情况，优化推荐模型。西瓜视频埋点需求是上报收藏按钮的点击事件 click_favorite，要求包含收藏影片的信息，所在的场景信息等。

1. 如果收藏事件发生在列表页，会上报如下的内容

```
{
  "event": "click_favorite",
  "params": {
    "video_id": "123", // 影片ID
    "video_type": 2, // 影片类型
    "page_name": "feed", // 当前页面
    "tab_name": "long_video" // 当前所在的底Tab
    "channel_name": "lvideo_recommend", // 当前所在的频道
  }
}
```

2. 如果收藏事件发生在详情页，会上报如下的内容

```
{
  "event": "click_favorite",
  "params": {
    "video_id": "123", // 影片ID
    "video_type": 2, // 影片类型
    "page_name": "detail", // 当前页面
    "from_page": "feed", // 来源页面
    "from_tab_name": "long_video" // 来源底Tab
    "from_channel_name": "lvideo_recommend", // 来源频道
  }
}
```

### 现有方案

前端用户交互的界面，通常有复杂的页面层级关系和跳转逻辑，为了准确记录埋点信息，满足上述埋点需求，主要有以下几种实现方案。

#### 直接传参

通过平台支持的参数传递方式，逐个定义并且读写参数；或者基于面向对象程序设计，对每个类添加相应的埋点参数，在类对象的关系中进行埋点参数传递。直接传参有非常显著的缺陷：

- 每增加一个参数，都需要写大量的重复代码，工程代码膨胀
- 模块间约定了很多埋点参数的协议，耦合程度高，难以维护
- 一些场景的嵌套层次深，经过很多层的参数传递，非常容易漏报埋点参数

#### 单例传参

上述问题有一种轻微缓解的办法，使用单例来进行埋点参数的访问。通过一个单例进行埋点参数的维护，由于单例提供了全局唯一访问入口，程序中的任何位置都能方便地读和写埋点参数。这种方式带来的好处是不需要在每个类都定义大量的埋点参数，只需要访问单例进行修改和读取。

会比前面的直接传参更简单，但这种方案治标不治本，同样有明显的弊端：

- 首先，无法解决列表页这种多实例场景的问题，比如一个推荐列表中有多个卡片，每个卡片的埋点参数都不一样，卡片的埋点参数还是需要自己传
- 单例的数据可能被多个位置写入，且一旦被覆盖就没法恢复，比如这样的路径：列表 -> 详情页 1 -> 相关推荐 -> 详情页 2，进到详情页 2 以后，单例的数据被覆盖了，这时候再回到详情页 1，获取到的埋点参数实际是详情页 2 的，导致埋点参数上报错误。
- 存放和清理的时机难以控制，清理早了会导致埋点参数缺失，忘记清理可能导致后面的埋点获取不到参数

#### 无埋点

无埋点是业界流行的一种埋点方案，所谓的“无埋点”、“全埋点”，是指埋点 SDK 通过编译时插桩、运行时反射或动态代理的方式，自动进行埋点事件的触发和上报，无须客户端工程师手动进行埋点开发工作。由产品经理、数据分析师等在埋点管理后台，使用 XPath 路径、页面视图 id 或者文本匹配等技术，定位到页面视图的位置，过滤出所需的数据。

此方案的优势很明显，客户端只需要一次性的接入，理论上能够搜集到所有页面、视图的曝光、点击等事件，无需客户端同学进行后续的埋点需求开发。

有这么好的事？为什么字节没有广泛使用？此方案的缺陷在于：

1. 仅能上报有限的简单事件类型，如页面视图曝光、点击等，无法完成复杂事件的上报，如一次支付行为的操作路径、结果、错误信息等
2. 无法自定义参数，主要指跳转的来源、所处的场景等上下文信息，无法满足复杂的数据分析和推荐模型所需的数据要求
3. 由产品经理、数据分析师等在埋点管理后台进行的事件录入，把复杂度从开发转嫁给了产品，消费成本较高
4. 对页面视图的稳定性有很高的要求，需要约定 id、文本、视图的层级，保持页面结构不变，如果客户端工程师因为一些新需求开发、性能优化等调整了视图结构，将会导致已录入的埋点失效，增加了额外的维护成本
5. 全场景的数据上报，可能产生大量的无用数据，消耗大量传输、存储、计算资源

### 西瓜视频的责任链方案

我们看下列表页的视图层级，会发现埋点参数恰好就分布在视图树的责任链中。

![图片](https://p3-juejin.byteimg.com/tos-cn-i-k3u1fbpfcp/2f54c6499d214e668c50a116f442af45~tplv-k3u1fbpfcp-zoom-1.image)

![图片](https://p3-juejin.byteimg.com/tos-cn-i-k3u1fbpfcp/c777c72524ee4fd3b75b7cbb05ab967f~tplv-k3u1fbpfcp-zoom-1.image)

比如在收藏按钮被点击时，只需要从收藏按钮的节点按照`卡片 -> 推荐频道 -> 放映厅Tab` 的顺序向上找，就能够拿到所有需要的参数了，并且这个上下级关系（视图树责任链）是已经客观存在的。

再来看看页面的跳转链路，跳转到详情页，有很多种路径。比如`推荐列表 -> 详情页`、`推荐列表 -> 选集 -> 详情页`，但逻辑上也是一个树状结构。

![图片](https://p3-juejin.byteimg.com/tos-cn-i-k3u1fbpfcp/4906defb017f474391880c5399951459~tplv-k3u1fbpfcp-zoom-1.image)

![图片](https://p3-juejin.byteimg.com/tos-cn-i-k3u1fbpfcp/e4056fcff5a84c75988f4fa60e7e6ab5~tplv-k3u1fbpfcp-zoom-1.image)

所以我们需要的埋点上下文参数，理论上都可以通过节点的关系找到。

### 参考文章

- [《西瓜客户端埋点实践：基于责任链的埋点框架》](https://mp.weixin.qq.com/s/iMn--4FNugtH26G90N1MaQ)